﻿$consoleId = $args[0]

$pipe = new-object System.IO.Pipes.NamedPipeClientStream("$consoleId")
$pipe.Connect()

$pipeReader = new-object System.IO.StreamReader($pipe)
$pipeWriter = new-object System.IO.StreamWriter($pipe)
$pipeWriter.AutoFlush = $true

$tempPath = [IO.Path]::GetTempFileName()
$envFile = "$tempPath.$consoleId.env"
$cdFile = "$tempPath.$consoleId.cd"

# global settings
$progressPreference = 'SilentlyContinue' # disable progress indicator

$chcpExists = $false
if (Get-Command "chcp.com" -ErrorAction SilentlyContinue) {
	$chcpExists = $true
}

function RunCmd($command) {
	

	if ($env:APPVEYOR_CONSOLE_RUN_CMD_IN_BATCH -eq 'false') {
		# single line
		$chcpCmd = ""
		if ($chcpExists) {
			$chcpCmd = "& chcp 65001 >nul"
		}
		& $env:comspec /c "@echo off $chcpCmd & $command && set > $envFile && cd > $cdFile"
	} else {
		# run in batch
		$scriptLines = @()
		$scriptLines += '@echo off'
		if ($chcpExists) {
			$scriptLines += 'chcp 65001 >nul'
		}
		$scriptLines += $command
		$scriptLines += 'set lastexitcode=%errorlevel%'
		$scriptLines += "set > $envFile"
		$scriptLines += "cd > $cdFile"
		$scriptLines += 'exit /b %lastexitcode%'

		$tempCmdFile = [IO.Path]::GetTempFileName() + '.cmd'
		[IO.File]::WriteAllLines($tempCmdFile, $scriptLines)

		& $env:comspec /c $tempCmdFile

		Remove-Item $tempCmdFile
	}
	
}

while ($true) {

	# read "cd"
	$currentDir = $pipeReader.ReadLine()
	cd $currentDir

	# save current path into file in case .cmd goes wrong
	Set-Content -Path $cdFile -Value $currentDir

	#Write-Host "Current dir: $currentDir"

	# read environment variables
	$varLines = @()
	while (($varLine = $pipeReader.ReadLine()) -ne '*** end env ***') {
		$idx = $varLine.IndexOf('=')
		if($idx -ne -1) {
			$varLines += $varLine
			[System.Environment]::SetEnvironmentVariable($varLine.Substring(0, $idx), $varLine.Substring($idx + 1))
		}
	}

	# save environment variables into file in case .cmd goes wrong
	Set-Content -Path $envFile -Value $varLines

	#Write-Host "Environment variables set"

	# read command
	$command = ""
	while (($cmdLine = $pipeReader.ReadLine()) -ne '*** end cmd ***') {
		$command += "$cmdLine`n"
	}

	#Write-Host "Command: $command"

	# execute command
	$Error.Clear()
	$global:LASTEXITCODE = $null

	$isCmd = $command.StartsWith('RunCmd ')

	$appveyor_exec_status = $true
	if (-not $isCmd) {
		$command += "`n`$appveyor_exec_status=`$?"
	}
		
	# execute command
	iex $command
		
	$exitCode = 0
	$errorResult = 'False'

	if ($LASTEXITCODE -ne $null -and $LASTEXITCODE -ne 0) {
		$exitCode = $LASTEXITCODE
	} elseif (-not $appveyor_exec_status) {
		$errorResult = 'True'
	}

	# write current directory
	if (-not $isCmd) {
		$currentDir = (pwd).path
	} else {
		$currentDir = Get-Content $cdFile
		Remove-Item $cdFile
	}

	$pipeWriter.WriteLine($currentDir)

	#Write-Host "Current dir written: $currentDir"

	# write environment variables
	if (-not $isCmd) {
		$envVars = [System.Environment]::GetEnvironmentVariables().GetEnumerator() | % { "$($_.Key)=$($_.Value)" }
	} else {
		$envVars = Get-Content $envFile
		Remove-Item $envFile
	}
	foreach ($varLine in $envVars) {
		$pipeWriter.WriteLine($varLine)
	}
	$pipeWriter.WriteLine("*** end env ***")

	#Write-Host "Env variables written"

	# write results
	#Write-Host "Command results to write: $exitCode|$errorResult"
	$pipeWriter.WriteLine("$exitCode|$errorResult")

	#Write-Host "Execution results written"
}