param
(
	[Parameter(Mandatory=$true)]
	[string]$vmName,

	[Parameter(Mandatory=$true)]
	[string]$snapshotPath
)

$ErrorActionPreference = "Stop"


# Check if VM exists
$vm = Get-VM $vmName -ErrorAction SilentlyContinue
if (-not $vm) {
    return
}


# Shutdown VM
try {
	Stop-VM -Name $vmName
}
catch {
	Write-Host "Cannot shutdown VM: $($_.Exception.Message)"
	exit 1
}


$imagesDir = [IO.Path]::GetDirectoryName($snapshotPath)
$exportDir = [IO.Path]::Combine($imagesDir, $vmName)
$vhdsDir = [IO.Path]::Combine($exportDir, 'Virtual Hard Disks')
$vhdPath = [IO.Path]::Combine($vhdsDir, "$vmName.vhdx")

# Create Images directory if not exists
if (-not (Test-Path $imagesDir)) {
	New-Item $imagesDir -Type Directory -Force | Out-Null
}

# Export VM
try {
	Export-VM -Name $vmName -Path $imagesDir
}
catch {
	Write-Host "Cannot export VM to $($imagesDir): $($_.Exception.Message)"
	exit 1
}

# Merge VHD
try {
	Merge-VHD -Path $vhdPath -Force
}
catch {
	Write-Host "Cannot merge VHD $($vhdPath): $($_.Exception.Message)"
	exit 1
}

# Move merged VHD to Images dir
try {
	$mergedVhdPath = (Get-ChildItem -Path $vhdsDir)[0].FullName
	Move-Item -Path $mergedVhdPath -Destination $snapshotPath
}
catch {
	Write-Host "Cannot move merged VHD $($mergedVhdPath) to $($snapshotPath): $($_.Exception.Message)"
	exit 1
}

# Delete export directory
try {
	Remove-Item -Path $exportDir -Recurse -Force
}
catch {
	Write-Host "Cannot delete export temp directory $($exportDir): $($_.Exception.Message)"
	exit 1
}